#!/usr/bin/env python

import sys

import numpy as np
import tables as tb

from ppflux.flux import atmospheric_honda_sarcevic
from km3pipe.tools import pdg2name

def ahs(mct, ene, zen):
    return atmospheric_honda_sarcevic(
        mc_type=mct,
        energy=ene,
        zenith=zen,
    )


honda_fname = '/home/mlotze/pkg/km3flux/km3flux/data/honda2015_frejus_solarmin.h5'
with tb.open_file(honda_fname) as h5:
    ebins = h5.root.energy_binlims[:]
    czbins = h5.root.cos_zen_binlims[:]

flavors = {14, -14, 12, -12}    # noqa
flux = {}
for flavor in flavors:
    flux[flavor] = np.zeros((czbins.shape[0] - 1, ebins.shape[0] - 1), dtype=float)

for i in range(ebins.shape[0] - 1):
    ene = np.mean((ebins[i], ebins[i+1]))
    for j in range(czbins.shape[0] - 1):
        cz = np.mean((czbins[j], czbins[j+1]))
        zen = np.arccos(cz)
        for flavor in flavors:
            flux[flavor][j, i] = ahs(flavor, ene, zen)

with tb.open_file('honda_sarcevic.h5', 'a') as h5:
    for mct, flux in flux.items():
        flav = pdg2name(mct)
        group = '/honda_sarcevic'
        h5.create_carray(group, flav, obj=flux, createparents=True)



#!/usr/bin/env python

from __future__ import division, absolute_import, print_function
import six

import h5py
import numpy as np
import pandas as pd


# define coszenith bins
nbins = 20
cos_zen_binlims = np.linspace(-1.0, 1.0, nbins + 1)
coszen_binsize = 2.0/nbins
coszen_low = cos_zen_binlims[:-1]
coszen_hi = cos_zen_binlims[1:]
cos_zen = coszen_low + 0.5 * coszen_binsize
bins = np.column_stack((coszen_low, coszen_hi))

prefix = '/home/mlotze/pkg/km3flux/km3flux'
dfs = []
for lo, hi in bins:
    fname = prefix + '/' + 'cos_%s_%s.csv' % (str(lo), str(hi))
    df = pd.read_table(fname, header=1, delim_whitespace=True)
    n_evts = len(df)
    df['cos_zen'] = lo + 0.5 * coszen_binsize
    dfs.append(df)

df = pd.concat(dfs)
energy = np.unique(df['Enu'])
en_low_log = np.log10(energy) - 0.025
en_high_log = np.log10(energy) + 0.025
en_low = np.power(10, en_low_log)
en_high = np.power(10, en_high_log)
en_binlims = np.ones(energy.shape[0] + 1)
en_binlims[:-1] = en_low
en_binlims[-1] = en_high[-1]
df = df.set_index(['cos_zen', 'Enu'])

# (Energy, cos) matrices
numu = df['NuMu'].unstack().values
numubar = df['NuMubar'].unstack().values
nue = df['NuE'].unstack().values
nuebar = df['NuEbar'].unstack().values

h5 = h5py.File('honda2015_frejus_solarmin.h5')
h5.create_dataset('energy', data=energy, compression="gzip", compression_opts=5)
h5.create_dataset('energy_binlims', data=en_binlims, compression="gzip", compression_opts=5)
h5.create_dataset('cos_zen', data=cos_zen, compression="gzip", compression_opts=5)
h5.create_dataset('cos_zen_binlims', data=cos_zen_binlims, compression="gzip", compression_opts=5)
h5.create_dataset('nu_mu', data=numu, compression="gzip", compression_opts=5)
h5.create_dataset('nu_mu_bar', data=numubar, compression="gzip", compression_opts=5)
h5.create_dataset('nu_e', data=nue, compression="gzip", compression_opts=5)
h5.create_dataset('nu_e_bar', data=nuebar, compression="gzip", compression_opts=5)
h5.close()



#!/usr/bin/env python
"""Utilities for effective Area computation.

All energies in GeV.
"""
from __future__ import division, absolute_import, print_function

import numpy as np


class Can(object):
    z_min = -117.2
    z_max = 139.5
    r_max = 205.402

    gen_vol = np.pi * (z_max - z_min) * np.square(r_max)

    @classmethod
    def contains(self, pos_r, pos_z):
        return (pos_r < self.r_max) & \
            (self.z_min < pos_z) & (pos_z < self.z_max)


def gen_events():
    from glob import glob
    infiles = glob('muon-CC_3-100*')
    n_files_with_muon_cc_3_100 = len(infiles)
    normalization = {       # noqa
        'muon-CC_3-100': 600 / n_files_with_muon_cc_3_100,
    }
    muon_cc_3_100_hist = []
    can = Can()     # noqa

    for fname in infiles:
        norm = 600   # muon-cc_3-100
        for evt in fname:
            nu = evt.mc_trks[0]
            pos_r = np.sqrt(np.square(nu.pos_x) + np.square(nu.pos_y))
            if Can.contains(pos_r, nu.pos_z):
                muon_cc_3_100_hist.fill(nu.E, nu.dir_z, wgt=1 / norm)


def effective_area(self, flux, w2, ene, n_gen_total,
                   angle_elem=4 * np.pi):
    seconds_in_a_year = 365.25 * 24 * 60 * 60
    n_gen_total = 1e6
    aeff = flux * w2 / (n_gen_total * angle_elem * seconds_in_a_year)
    return aeff



"""Utilities to compute event weights.

Most important is the make_weights function, to compute weights for mixed
flavors (e.g. neutrinos + mupage).
"""
from __future__ import division, absolute_import, print_function

import numpy as np
from km3pipe.mc import pdg2name

from km3flux.flux import e2flux


def nu_wgt(w2, n_gen, adjust_orca_overlap=False, energy=None):
    """Get neutrino weight, optionally adjust for ORCA energy overlap.

    Aka crazy Joao hack:

        // If energy in overlap region, divide by 2
        if(nu.E>3 && nu.E<5) weight /= 2;
   """
    wgt = w2 / n_gen
    if adjust_orca_overlap and (energy is None):
        raise ValueError("When correcting for orca overlap, you need "
                         "to specify the energy.")
    if adjust_orca_overlap and (energy is not None):
        overlap_mask = (3 <= energy) & (energy <= 5)
        wgt[overlap_mask] /= 2
    return wgt


def atmu_wgt(livetime_sec, fill_blank=False, fill=60.0):
    """Compute mupage weight."""
    if fill_blank:
        livetime_sec = np.full_like(livetime_sec, fill)
    out = 1 / livetime_sec
    return out


def make_weights(w2, n_gen, livetime_sec, is_neutrino,
                 adjust_orca_overlap=False, energy=None, fix_atmu=False):
    """Generate weights for events of mixed flavor.

    This assumes that all arrays have the same length. If you have mixed
    neutrino + atmu events, just pad the unused fields (e.g. livetime for
    neutrinos, w2 for atmu) with anything, they will be ignored.

    Of course, if your data has only neutrinos/mupage, use the dedicated
    methods for them instead: `nu_wgt` and `atmu_wgt`.

    Parameters
    ==========
    w2: array-like
    n_gen: scalar or array-like
        number ov generated events *in the entire production*
        for gSeaGen, this is already n_gen.
        For GenHen, this is `n_gen * n_files`
    livetime_sec: scalar or array-like
        livetime *in seconds*
    is_neutrino: boolean array
    adjust_orca_overlap: bool, default=False
        Some Orca productions (2016) have energy overlap, between 3-5 GeV.
        If set True, you need to pass in an energy.
    energy: array, default=None
        If `adjust_orca_overlap` is set, you need to pass this.
    """
    wgt = np.ones(len(w2))
    wgt[is_neutrino] = nu_wgt(w2[is_neutrino], n_gen[is_neutrino],
                              adjust_orca_overlap=adjust_orca_overlap,
                              energy=energy)
    wgt[~is_neutrino] = atmu_wgt(livetime_sec[~is_neutrino],
                                 fill_blank=fix_atmu)
    return wgt


def strange_flavor_to_mupage(flav, fill='mu+'):
    mask = ((flav == 'N/A') | (flav == 'n'))
    flav[mask] = fill
    return flav


def add_flavor(df, fix_strange_flavor=True):
    def t2f(row):
        return pdg2name(row['type'])

    flavor = df.apply(t2f, axis=1).astype('str')
    if fix_strange_flavor:
        flavor = strange_flavor_to_mupage(flavor)
    return flavor


def add_weights_and_fluxes(df, **kwargs):
    """Add weights + common fluxes."""
    for k in ['weight_w2', 'n_events_gen', 'livetime_sec',
              'is_neutrino', 'energy']:
        assert k in df.columns
    df['flavor'] = add_flavor(df)
    df['wgt'] = make_weights(df.weight_w2, df.n_events_gen, df.livetime_sec,
                             df.is_neutrino, adjust_orca_overlap=True,
                             energy=df.energy, **kwargs)
    df['e2flux'] = e2flux(df.energy)
    df = make_atmo_weight(df)
    return df


def make_atmo_weight(df_):
    df_['wgt_atmo'] = df_.wgt.copy()
    hon = df_.honda[df_.flavor != 'mu+']
    df_.loc[df_.flavor != 'mu+', 'wgt_atmo'] *= hon
    return df_



#!/usr/bin/env python
"""Parse the averaged tables."""

from __future__ import division, absolute_import, print_function
import six

import h5py
import numpy as np
import pandas as pd

prefix = '/home/mlotze/pkg/km3flux/km3flux/data'
fname = prefix + '/' + 'all.csv'
df = pd.read_table(fname, header=0, delim_whitespace=True)
n_evts = len(df)

energy = np.unique(df['Enu'])
en_low_log = np.log10(energy) - 0.025
en_high_log = np.log10(energy) + 0.025
en_low = np.power(10, en_low_log)
en_high = np.power(10, en_high_log)
en_binlims = np.ones(energy.shape[0] + 1)
en_binlims[:-1] = en_low
en_binlims[-1] = en_high[-1]
df = df.set_index(['Enu'])

# (Energy, cos) matrices
numu = df['NuMu'].values
numubar = df['NuMubar'].values
nue = df['NuE'].values
nuebar = df['NuEbar'].values

h5 = h5py.File('data/honda2015_frejus_solarmin.h5')
h5.create_group('averaged')
#h5.create_dataset('energy', data=energy, compression="gzip", compression_opts=5)
#h5.create_dataset('energy_binlims', data=en_binlims, compression="gzip", compression_opts=5)
h5.create_dataset('averaged/nu_mu', data=numu, compression="gzip", compression_opts=5)
h5.create_dataset('averaged/nu_mu_bar', data=numubar, compression="gzip", compression_opts=5)
h5.create_dataset('averaged/nu_e', data=nue, compression="gzip", compression_opts=5)
h5.create_dataset('averaged/nu_e_bar', data=nuebar, compression="gzip", compression_opts=5)
h5.close()



from __future__ import division, absolute_import, print_function

import os.path

import numpy as np
import pandas as pd
import h5py

import km3flux


DATADIR = os.path.dirname(km3flux.__file__) + '/data'
HONDAFILE = DATADIR + '/honda2015_frejus_solarmin.h5'
WIMPSIM_FILE = DATADIR + '/wimpsim_1d.h5'
# DM_GC_FILE = DATADIR + '/gc_spectra.h5'
DM_GC_FILE = DATADIR + '/cirelli_gc.h5'
DM_GC_MASSES = {'100000', '260', '100', '5000', '360', '200', '10', '1000',
                '750', '30000', '2000', '50', '10000', '180', '500', '150',
                '3000', '25', '90', '1500'}
DM_GC_FLAVORS = {'anu_mu', 'nu_mu'}
DM_GC_CHANNELS = {'b', 'mu', 'tau', 'w'}

DM_SUN_FILE = DATADIR + '/sun_spectra.h5'
DM_SUN_MASSES = {'1000', '100', '10', '1500', '176', '150', '2000', '200',
                 '250', '25', '3000', '350', '5000', '500', '50', '750'}
DM_SUN_FLAVORS = {'anu_mu', 'nu_mu'}
DM_SUN_CHANNELS = {'11', '8', '5'}

DM_SUN_CHAN_TRANS = {'8': 'w', '11': 'tau', '5': 'b'}
DM_SUN_CHAN_TRANS_INV = {v: k for k, v in DM_SUN_CHAN_TRANS.items()}

WIMPSIM_CHANNELS = {
    1: 'd d-bar',
    2: 'u u-bar',
    3: 's s-bar',
    4: 'c c-bar',
    5: 'b b-bar',
    6: 't t-bar',
    7: 'glue glue',
    8: 'W+ W-',
    9: 'Z0 Z0',
    10: 'mu- mu+',
    11: 'tau- tau+',
    12: 'nu_e nu_e-bar',
    13: 'nu_mu nu_mu-bar',
    14: 'nu_tau nu_tau-bar',
}
WIMPSIM_FLAVORS = ['nu_e', 'anu_e', 'nu_mu', 'anu_mu', 'nu_tau', 'anu_tau']
WIMPSIM_INTERESTING_CHANNELS = {8, 11, 5}
WIMPSIM_INTERESTING_CHANNELS.update(
    {WIMPSIM_CHANNELS[c] for c in WIMPSIM_INTERESTING_CHANNELS})
WIMPSIM_MASSES = [5.00e+02, 3.00e+00, 1.50e+03, 5.00e+03, 6.00e+00, 2.00e+03,
                  1.50e+02, 7.50e+03, 2.50e+02, 9.12e+01, 1.00e+04, 1.00e+03,
                  1.00e+01, 3.50e+02, 2.00e+02, 1.00e+02, 1.76e+02, 2.50e+01,
                  7.50e+02, 8.03e+01, 3.00e+03, 5.00e+01]


def dm_gc_spectrum(flavor='nu_mu', channel='w', mass='100', full_lims=False):
    """Dark Matter spectra by M. Cirelli."""
    mass = str(mass)
    if mass not in DM_GC_MASSES:
        raise KeyError("Mass '{}' not available.".format(mass))
    if flavor not in DM_GC_FLAVORS:
        raise KeyError("Flavor '{}' not available.".format(flavor))
    if channel not in DM_GC_CHANNELS:
        raise KeyError("Channel '{}' not available.".format(channel))

    fname = DM_GC_FILE
    with h5py.File(fname, 'r') as h5:
        gr = h5[flavor][channel][mass]
        counts = gr['entries'][:]
        bins = gr['binlims'][:]
    if not full_lims:
        bins = bins[:-1]
    return counts, bins


def dm_sun_spectrum(flavor='nu_mu', channel='w', mass='100', full_lims=False):
    """Dark Matter spectra by M. Cirelli."""
    chan_num = DM_SUN_CHAN_TRANS_INV[channel]
    mass = str(mass)
    if mass not in DM_SUN_MASSES:
        raise KeyError("Mass '{}' not available.".format(mass))
    if flavor not in DM_SUN_FLAVORS:
        raise KeyError("Flavor '{}' not available.".format(flavor))
    if chan_num not in DM_SUN_CHANNELS:
        raise KeyError("Channel '{}' not available.".format(channel))

    fname = DM_SUN_FILE
    with h5py.File(fname, 'r') as h5:
        gr = h5[flavor][chan_num][mass]
        counts = gr['entries'][:]
        bins = gr['binlims'][:]
    if not full_lims:
        bins = bins[:-1]
    return counts, bins


# WIMPSIM
# n_lines_2d_per_flavor = 50
# n_z_bins_1d = 100
# z_bins_1d = np.linspace(0.005, 0.995, n_z_bins_1d)
# n_z_bins_2d = n_lines_2d_per_flavor
# z_bins_2d = np.linspace(0.01, 0.99, n_z_bins_2d)
#
# n_theta_bins = 91
# theta_first = np.linspace(0.1, 9.9, 50)
# theta_second = np.linspace(10.25, 29.75, 40)
# # last bin is >30
# theta_last = [30., ]
# theta_bins = np.concatenate([theta_first, theta_second, theta_last])
# theta_bins


def wimpsim_parse_fname(fname):
    """Parse wimpsim filename.

    Returns
    -------
    parent_mass, channel, n_dimensions
    """
    # example: './data/we-m10000-ch11-su-1D-diff-f1.dat'
    _, mass, chan, _, dim, _, _ = os.path.basename(fname).split('-')
    # 'm10000'
    mass = float(mass[1:])
    # 'ch11'
    chan = int(chan[2:])
    # '1D' / '2D'
    ndim = int(dim[0])
    return mass, chan, ndim


def wimpsim_read_file(fname):
    mass, chan, ndim = wimpsim_parse_fname(fname)
    if ndim == 1:
        df = wimpsim_read_1d(fname)
    else:
        raise NotImplementedError(
            'Only 1d tables supported, got ndim={}. Sorry!'.format(ndim))
    df['mass'] = mass
    df['energy'] = df['mass'] * df['z']
    df['chan_num'] = chan
    #df['channel'] = CHANNELS[chan]
    return df


def wimpsim_read_1d(fname):
    n_z_bins_1d = 100
    z_bins_1d = np.linspace(0.005, 0.995, n_z_bins_1d)
    df = pd.read_csv(fname, delim_whitespace=True,
                     comment='#', header=None,
                     )
    df = df.T
    df.columns = WIMPSIM_FLAVORS
    df['z'] = z_bins_1d
    return df



#!/usr/bin/env python
# vim:set ts=4 sts=4 sw=4 et:

from setuptools import setup
from km3flux import __version__

setup(
    name='km3flux',
    version=__version__,
    description='Atmospheric Neutrino Fluxes',
    url='http://git.km3net.de/km3py/km3flux',
    author='Moritz Lotze',
    author_email='mlotze@km3net.de',
    license='MIT',
    packages=['km3flux', ],
    include_package_data=True,
    install_requires=[
        'numpy',
        'scipy>=0.19',
        'h5py',
        'docopt',
        'matplotlib>=2.0',
        'pandas',
        'tables',
        'km3pipe[full]>=7.3.2'
    ]
)



"""Assorted Fluxes, in  (m^2 sec sr GeV)^-1"""

import logging
from six import string_types

import h5py
import numpy as np
import pandas as pd
from scipy.integrate import romberg, simps
from scipy.interpolate import splrep, splev, RectBivariateSpline

from km3pipe.mc import name2pdg, pdg2name
from km3flux.data import (HONDAFILE, DM_GC_FLAVORS, DM_GC_CHANNELS,     # noqa
                          DM_GC_MASSES, DM_GC_FILE, WIMPSIM_FILE,
                          WIMPSIM_FLAVORS, WIMPSIM_INTERESTING_CHANNELS,
                          WIMPSIM_MASSES,
                          # dm_gc_spectrum, dm_sun_spectrum,
                          # DM_SUN_FLAVORS, DM_SUN_CHANNELS, DM_SUN_MASSES
                         )
from km3pipe.tools import issorted
from km3pipe.plot import bincenters

FLAVORS = ('nu_e', 'anu_e', 'nu_mu', 'anu_mu')
MCTYPES = [name2pdg(name) for name in FLAVORS]

logger = logging.getLogger(__name__)


class BaseFlux(object):
    """Base class for fluxes.

    Methods
    =======
    __call__(energy, zenith=None)
        Return the flux on energy, optionally on zenith.
    integrate(zenith=None, emin=1, emax=100, **integargs)
        Integrate the flux via romberg integration.
    integrate_samples(energy, zenith=None, emin=1, emax=100)
        Integrate the flux from given samples, via simpson integration.

    Example
    =======
    >>> zen = np.linspace(0, np.pi, 5)
    >>> ene = np.logspace(0, 2, 5)

    >>> from km3flux.flux import MyFlux
    >>> flux = MyFlux(flavor='nu_mu')

    >>> flux(ene)
    array([6.68440000e+01, 1.83370000e+01, 4.96390000e+00,
           1.61780000e+00, 5.05350000e-01,])
    >>> flux(ene, zen)
    array([2.29920000e-01, 2.34160000e-02, 2.99460000e-03,
           3.77690000e-04, 6.87310000e-05])
    """
    def __init__(self, **kwargs):
        pass

    def __call__(self, energy, zenith=None, interpolate=True):
        logger.debug("Interpolate? {}".format(interpolate))
        energy = np.atleast_1d(energy)
        logger.debug("Entering __call__...")
        if zenith is None:
            logger.debug("Zenith is none, using averaged table...")
            return self._averaged(energy, interpolate=interpolate)
        zenith = np.atleast_1d(zenith)
        if len(zenith) != len(energy):
            raise ValueError("Zenith and energy need to have the same length.")
        logger.debug("Zenith available, using angle-dependent table...")
        return self._with_zenith(energy=energy, zenith=zenith,
                                 interpolate=interpolate)

    def _averaged(self, energy, interpolate=True):
        logger.debug("Interpolate? {}".format(interpolate))
        raise NotImplementedError

    def _with_zenith(self, energy, zenith, interpolate=True):
        logger.debug("Interpolate? {}".format(interpolate))
        raise NotImplementedError

    def integrate(self, zenith=None, emin=1, emax=100, interpolate=True, **integargs):
        logger.debug("Interpolate? {}".format(interpolate))
        return romberg(self, emin, emax, vec_func=True, **integargs)

    def integrate_samples(self, energy, zenith=None, emin=1, emax=100,
                          interpolate=True, **integargs):
        logger.debug("Interpolate? {}".format(interpolate))
        energy = np.atleast_1d(energy)
        mask = (emin <= energy) & (energy <= emax)
        energy = energy[mask]
        if zenith:
            logger.debug("Zenith available, using angle-dependent table...")
            zenith = np.atleast_1d(zenith)
            zenith = zenith[mask]
        flux = self(energy, zenith=zenith, interpolate=interpolate)
        return simps(flux, energy, **integargs)


class PowerlawFlux(BaseFlux):
    """E^-gamma flux."""
    def __init__(self, gamma=2, scale=1e-4):
        self.gamma = gamma
        self.scale = scale

    def _averaged(self, energy, interpolate=True):
        return self.scale * np.power(energy, -1 * self.gamma)

    def integrate(self, zenith=None, emin=1, emax=100, **integargs):
        """Compute analytic integral instead of numeric one."""
        if np.around(self.gamma, decimals=1) == 1.0:
            return np.log(emax) - np.log(emin)
        num = np.power(emax, 1 - self.gamma) - np.power(emin, 1 - self.gamma)
        den = 1.0 - self.gamma
        return self.scale * (num / den)


class Honda2015(BaseFlux):
    """
    Get Honda 2015 atmospheric neutrino fluxes.

    Whitepaper at https://arxiv.org/abs/1502.03916.

    Flux table downloaded from http://www.icrr.u-tokyo.ac.jp/~mhonda/

    Flux at Frejus site, no mountain, solar minimum.

    Units are (1 / m^2 sec sr GeV)

    Methods
    =======
    __init__(flavor='nu_mu')
        Load flux table for the given neutrino flavor.
    __call__(energy, zenith=None)
        Return the flux on energy, optionally on zenith.
    integrate(zenith=None, emin=1, emax=100, **integargs)
        Integrate the flux via omberg integration.
    integrate_samples(energy, zenith=None, emin=1, emax=100)
        Integrate the flux from given samples, via simpson integration.
    """
    def __init__(self, flavor='nu_mu'):
        self.table = None
        self.avtable = None
        filename = HONDAFILE
        if flavor not in FLAVORS:
            raise ValueError("Unsupported flux '{}'".format(flavor))
        with h5py.File(filename, 'r') as h5:
            self.energy_bins = h5['energy_binlims'][:]
            self.cos_zen_bins = h5['cos_zen_binlims'][:]
            self.table = h5[flavor][:]
            self.avtable = h5['averaged/' + flavor][:]
        # adjust upper bin for the case zenith==0
        self.cos_zen_bins[-1] += 0.00001
        self.energy_bincenters = bincenters(self.energy_bins)
        self.cos_zen_bincenters = bincenters(self.cos_zen_bins)
        assert issorted(self.cos_zen_bincenters)
        assert issorted(self.energy_bincenters)
        assert self.avtable.shape == self.energy_bincenters.shape
        assert self.table.shape[0] == self.cos_zen_bincenters.shape[0]
        assert self.table.shape[1] == self.energy_bincenters.shape[0]
        self.avinterpol = splrep(
            self.energy_bincenters,
            self.avtable,
        )
        self.interpol = RectBivariateSpline(
            self.cos_zen_bincenters,
            self.energy_bincenters,
            self.table
        )

    def _averaged(self, energy, interpolate=True):
        logger.debug("Interpolate? {}".format(interpolate))
        if not interpolate:
            fluxtable = self.avtable
            ene_bin = np.digitize(energy, self.energy_bins)
            ene_bin = ene_bin - 1
            return fluxtable[ene_bin]
        else:
            flux = splev(energy, self.avinterpol)
            return flux

    def _with_zenith(self, energy, zenith, interpolate=True):
        logger.debug("Interpolate? {}".format(interpolate))
        energy = np.atleast_1d(energy)
        zenith = np.atleast_1d(zenith)
        cos_zen = np.cos(zenith)
        if not interpolate:
            fluxtable = self.table
            ene_bin = np.digitize(energy, self.energy_bins)
            zen_bin = np.digitize(cos_zen, self.cos_zen_bins)
            ene_bin = ene_bin - 1
            zen_bin = zen_bin - 1
            return fluxtable[zen_bin, ene_bin]
        else:
            flux = self.interpol.ev(cos_zen, energy)
            return flux


class HondaSarcevic(BaseFlux):
    """
    Get Honda + Sarcevic atmospheric neutrino fluxes.
    """
    def __init__(self, flavor='nu_mu'):
        self.table = None
        self.avtable = None
        filename = HONDAFILE
        if flavor not in FLAVORS:
            raise ValueError("Unsupported flux '{}'".format(flavor))
        with h5py.File(filename, 'r') as h5:
            self.energy_bins = h5['energy_binlims'][:]
            self.cos_zen_bins = h5['cos_zen_binlims'][:]
            self.table = h5['honda_sarcevic/' + flavor][:]
        # adjust upper bin for the case zenith==0
        self.cos_zen_bins[-1] += 0.00001

    def _averaged(self, energy, interpolate=True):
        raise NotImplementedError("Supports only zenith dependent flux!")

    def _with_zenith(self, energy, zenith, interpolate=True):
        fluxtable = self.table
        cos_zen = np.cos(zenith)
        ene_bin = np.digitize(energy, self.energy_bins)
        zen_bin = np.digitize(cos_zen, self.cos_zen_bins)
        ene_bin = ene_bin - 1
        zen_bin = zen_bin - 1
        return fluxtable[zen_bin, ene_bin]


class DarkMatterFlux(BaseFlux):
    """
    Get Dark Matter WimpWimp->foo->nu spectra.

    Units are ``d/d n_particles / cm^2 sec sr``.

    >>> from km3flux import DarkMatterFlux
    >>> flux = DarkMatterFlux('anu_mu')
    >>> flux(ene)
    array([  6.68440000e+01,   1.83370000e+01,   4.96390000e+00,
         1.61780000e+00,   5.05350000e-01,   2.29920000e-01,
         2.34160000e-02,   2.99460000e-03,   3.77690000e-04,
         6.87310000e-05,   1.42550000e-05])

    Methods
    =======
    __init__(source='gc', flavor='nu_mu', channel='w', mass=1000)
        Load flux table for the given neutrino flavor.
    __call__(energy, zenith=None, interpolate=True)
        Return the flux on energy, optionally on zenith.
    integrate(zenith=None, emin=1, emax=100, **integargs)
        Integrate the flux via romberg integration.
    integrate_samples(energy, zenith=None, emin=1, emax=100)
        Integrate the flux from given samples, via simpson integration.

    Example
    =======
    >>> from km3flux import DarkMatterFlux
    >>> print(DarkMatterFlux.flavors)
    >>> flux = DarkMatterFlux(flavor='anu_mu', channel='w', mass=1000)
    >>> ene = np.geomspace(1, 100, 11)
    >>> print(flux(ene, interpolate=True))
    """
    flavors = DM_GC_FLAVORS
    channels = DM_GC_CHANNELS
    masses = DM_GC_MASSES

    def __init__(self, source='gc', flavor='nu_mu', channel='w', mass=1000):
        """
        Parameters
        ==========
        source: string, optional [default: 'gc']
            Object where the WimpWimp annihilates. Currently only 'gc'
        flavor: string, optional [default: 'nu_mu']
            Neutrino flavor. See available flavors stored in
            ``DarkMatterFlux.flavors``
        channel: string, optional [default: 'w']
            WimpWimp annihilation channel (WimpWimp -> foofoo).
            See available channels stored in ``DarkMatterFlux.channels``
        mass: int, optional [default: 1000]
            WimpWimp parent mass.
            See available masses stored in ``DarkMatterFlux.masses``
        """
        if source == 'gc':
            fname = DM_GC_FILE
        else:
            raise NotImplementedError('Only GC supported so far!')
        self.mass = mass
        self.channel = channel
        self.flavor = flavor

        tab = pd.read_hdf(fname)
        tab = tab[(tab.flavor == flavor) & (tab.mass_dm == mass)]
        self.flux = tab[channel].values
        self.x_energy = tab['energy']
        self.avinterpol = splrep(
            self.x_energy,
            self.flux,
        )

    @property
    def points(self):
        return self.x_energy, self.flux

    def _check_energy(self, ene, epsilon=0.0001):
        if np.any(ene > self.mass + epsilon):
            raise ValueError('Energies exceed parent mass!')
        return ene

    def __call__(self, energy, interpolate=True):
        energy = np.atleast_1d(energy)
        energy = self._check_energy(energy)
        flux = splev(energy, self.avinterpol)
        return flux

    def _averaged(self, *args, **kwargs):
        return self(*args, **kwargs)

    def _with_zenith(self, energy, zenith, interpolate=True):
        logger.debug("Interpolate? {}".format(interpolate))
        logging.warning('No zenith dependent flux implemented! '
                        'Falling back to averaged flux.')
        return self._averaged(self, energy)


class AllFlavorFlux():
    """Get mixed-flavor fluxes.

    Methods
    =======
    __init__(fluxclass='Honda2015')

    __call__(energy, zenith=None, interpolate=True)
        Return the flux on energy, optionally on zenith.
    """
    fluxmodels = {
        'Honda2015': Honda2015,
        'HondaSarcevic': HondaSarcevic,
    }

    def __init__(self, fluxclass='Honda2015'):
        if isinstance(fluxclass, string_types):
            fluxclass = self.fluxmodels[fluxclass]
        self.flux_flavors = {}
        for flav in FLAVORS:
            self.flux_flavors[flav] = fluxclass(flav)

    def __call__(self, energy, zenith=None, flavor=None, mctype=None,
                 interpolate=True):
        """mctype is ignored if flavor is passed as arg."""
        if mctype is None and flavor is None:
            raise ValueError("Specify either mctype(int) or flavor(string)")
        if flavor is None:
            mctype = pd.Series(np.atleast_1d(mctype))
            flavor = mctype.apply(pdg2name)
        flavor = np.atleast_1d(flavor)
        energy = np.atleast_1d(energy)
        out = np.zeros_like(energy)
        if zenith is not None:
            zenith = np.atleast_1d(zenith)
        for flav in np.unique(flavor):
            where = flavor == flav
            if zenith is not None:
                loczen = zenith[where]
            flux = self.flux_flavors[flav](energy[where], loczen,
                                           interpolate=interpolate)
            out[where] = flux
        return out


class WimpSimFlux(BaseFlux):
    """WimpSim neutrino flux from Sun.

    http://wimpsim.astroparticle.se/data.html

    `z: nu mass / wimp mass`

    `theta: angle towards center of sun`

    `z=E_nu/m_WIMP`

    2d layout: `d^2N/dz dtheta`

    1d: `dN/dz`

    Neutrinos at detector (icecube), the units are `cm^-2 annihilation^-1`.

    we-res-jan2013-data.tar.gz

    All energies in GeV.
    """
    def __init__(self, filename=None):
        if filename is None:
            filename = WIMPSIM_FILE
        self.tab = pd.read_hdf(filename, '/flat')

    @property
    def points(self):
        return self.x_energy, self.dnde

    def _check_energy(self, ene, epsilon=0.0001):
        if np.any(ene > self.mass + epsilon):
            raise ValueError('Energies exceed parent mass!')
        return ene

    def __call__(self, energy, interpolate=True,
                 flavor='nu_mu', mass=1000.0, channel='W+ W-',):
        self.mass = mass
        tab = self.tab[
            (self.tab.chan_num == 11) &
            (np.isclose(self.tab.mass, 1000.0))
        ]
        dnde = tab[flavor]
        self.x_energy = tab['energy']
        self.avinterpol = splrep(
            self.x_energy,
            dnde,
        )
        energy = np.atleast_1d(energy)
        energy = self._check_energy(energy)
        dnde = splev(energy, self.avinterpol)
        return self.dnde2flux(dnde, self.mass)

    def _averaged(self, *args, **kwargs):
        return self(*args, **kwargs)

    def _with_zenith(self, energy, zenith, interpolate=True):
        raise NotImplementedError

    @staticmethod
    def dnde2flux(dnde, mass):
        """Expects Neutrinos at detector,

        Input units are `cm^-2 annihilation^-1`.

        Output is / (m^2 sec sr GeV)
        """
        # dN/dz comes out of WS
        # y = dN/dz = MDM * (dN/dE)
        # gamm = (1e14/sec) * (100GeV/m_dm)^2 * (4*pi*r_suntoearth)
        # -> [GeV m^2 sr s/yr]
        # geom fact in wimpsim = 1/(4pi * r_sun_to_earth)
        # -> geom fact already included (if using "yields at detector" table)
        # -> gamma = 1e14 * (100/m)^2
        # i don't get the 1e9 ???
        # bah whatever
        flux = (dnde / mass) * 1e14 * 1e9 * np.square(100 / mass)
        flux *= 1e4     # cm2 -> m2
        # gseagen:
        # pointsource, so the I_0 = 1 (no 1/sr)
        return flux


def add_honda(df):
    flav = df.flavor.iloc[0]    # assume all equal (e.g. inside `groupby(flavor)`)
    # (m^2 sec sr GeV)^-1
    honda = Honda2015(flav)(energy=df.energy, zenith=df.zenith,
                            interpolate=True)
    df['honda'] = honda
    return df


def add_wimp(df, mass=1000.0, channel='W+ W-'):
    flav = df.flavor.iloc[0]    # assume all equal (e.g. inside `groupby(flavor)`)
    # (m^2 sec sr GeV)^-1
    wimp = WimpSimFlux()(flavor=flav, energy=df.energy,
                         mass=mass, channel=channel,
                         interpolate=True)
    df['wimpsim_{}_{}'.format(channel, mass)] = wimp
    return df


def apply_honda(df):
    """Add Honda flux to a dataframe."""
    return df.groupby(df.flavor).apply(add_honda)


def apply_wimp(df, **kwargs):
    """Add WimpSim flux to a dataframe."""
    return df.groupby(df.flavor).apply(add_wimp, **kwargs)
